C PACKAGE 'CRT' FOR COMPLETE RAY TRACING
C
C BY VLASTISLAV CERVENY, LUDEK KLIMES, IVAN PSENCIK
C
C THIS PACKAGE CONSISTS OF:
C     CRT...  MAIN PROGRAM CONTROLING THE COMPLETE RAY TRACING.
C     UNIT... SUBROUTINE DESIGNED TO CONTROL CONNECTING AND
C             DISCONNECTING THE DATA FILES TO LOGICAL UNITS, AND TO
C             DETERMINE THE LOGICAL UNITS FROM WHICH THE GIVEN DATA SETS
C             ARE TO BE READ.
C
C=======================================================================
C
C MAIN PROGRAM CRT FOR COMPLETE RAY TRACING
C
C THIS PROGRAM READS THE INPUT DATA AND THEN CONTROLS THE COMPLETE RAY
C TRACING OF THE SPECIFIED ELEMENTARY WAVES.
C
C INPUT DATA -  FILE(1)...MAIN DATA SET CRT (NO NAME):
C     THE NAME OF THE MAIN INPUT DATA FILE IS GIVEN BY THE FIRST
C     EXECUTIVE STATEMENT OF THE MAIN PROGRAM.  IT IS BLANK IN THE
C     ORIGINAL VERSION.  THIS MAIN DATA FILE CONTAINS THE NAMES OF OTHER
C     INPUT FILES AND THE NAME OF THE OUTPUT LOG FILE.  IT MAY OR MAY
C     NOT CONTAIN OTHER INPUT DATA.  THE NAMES OF THE OUTPUT FILES WITH
C     THE COMPUTED QUANTITIES (C.R.T.5.5) ARE SPECIFIED IN THE
C     SUBROUTINE PACKAGE 'WRIT'.
C     THE DATA ARE READ IN BY THE LIST DIRECTED INPUT (FREE FORMAT).  IN
C     THE LIST OF INPUT DATA BELOW, EACH NUMBERED PARAGRAPH INDICATES
C     THE BEGINNING OF A NEW INPUT OPERATION (NEW READ STATEMENT).  ALL
C     INPUT VARIABLES ARE OF THE TYPE CHARACTER.  ONLY THE FIRST 80
C     CHARACTERS OF THE STRINGS ARE SIGNIFICANT.
C (1) TEXT
C     THE STRING DESCRIBING THE INPUT DATA.
C (2) FILE(2)...DATA SET MODEL
C     THE STRING CONTAINING THE NAME OF THE FILE WITH THE INPUT DATA FOR
C     THE MODEL.  THE DATA WILL BE READ IN BY THE SUBROUTINE MODEL1.
C (3) FILE(3)...DATA SET DCRT
C     THE STRING CONTAINING THE NAME OF THE FILE WITH THE INPUT DATA FOR
C     THE COMPLETE RAY TRACING.  THE DATA WILL BE READ IN BY THE
C     SUBROUTINE RAY1.
C (4) FILE(4)...DATA SET INIT
C     THE STRING CONTAINING THE NAME OF THE FILE WITH THE INPUT DATA
C     SPECIFYING THE INITIAL CONDITIONS FOR RAYS.  THE DATA WILL BE READ
C     IN BY THE SUBROUTINE INIT1.
C (5) FILE(5)...DATA SET CODE
C     THE STRING CONTAINING THE NAME OF THE FILE WITH THE CODES OF
C     ELEMENTARY WAVES.  THE DATA WILL BE READ IN BY THE SUBROUTINE
C     CODE1.
C (6) FILE(5)...DATA SET RPAR
C     THE STRING CONTAINING THE NAME OF THE FILE WITH THE DATA
C     SPECIFYING THE TAKE-OFF PARAMETERS OF THE REQUIRED RAYS.  THE DATA
C     WILL BE READ IN BY THE SUBROUTINE RPAR1.
C (7) FILE(6)...DATA SET WRIT
C     THE STRING CONTAINING THE NAME OF THE FILE SPECIFYING THE NAMES OF
C     THE OUTPUT FILES WITH THE COMPUTED QUANTITIES.  THESE DATA WILL BE
C     READ BY THE SUBROUTINE WRIT1.
C (8) FILE(6)...DATA SET LOG
C     THE STRING CONTAINING THE NAME OF THE OUTPUT LOG FILE.  THE DATA
C     WILL BE WRITTEN BY THE SUBROUTINES WRIT1, WRIT2, WRIT4 AND WRIT5.
C THE FILENAMES FILE(1) TO FILE(8) NEED NOT BE MUTUALLY DIFFERENT,
C SEVERAL DATA SETS MAY BE READ FROM (OR WRITTEN TO) THE SAME DATA FILE.
C EACH DATA FILE IS CLOSED WHEN READ OVER, AND ITS LOGICAL UNIT NUMBER
C MAY BE CONNECTED TO ANOTHER FILE TO BE OPENED.  WHEN MORE THAN ONE
C ELEMENTARY WAVE IS COMPUTED, IT IS NOT RECOMMENDED TO WRITE THE LOG
C OUTPUT DATA SET TO THE FILE CONTAINING THE CODE, RPAR OR WRIT DATA
C SET.
C
C SUBROUTINES REFERENCED:
      EXTERNAL MODEL1,CODE1,RAY1,RAY2,INIT1,INIT2,RPAR1,RPAR2,RPAR4
      EXTERNAL WRIT1,WRIT2,WRIT4,WRIT5,UNIT
C     MODEL1..SUBROUTINE PACKAGE 'MODEL'.
C     CODE1...SUBROUTINE PACKAGE 'CODE'.
C     RAY1, RAY2... SUBROUTINE PACKAGE 'RAY'.
C     INIT1, INIT2... SUBROUTINE PACKAGE 'INIT'.
C     RPAR1, RPAR2, RPAR4... SUBROUTINE PACKAGE 'RPAR'.
C     WRIT1, WRIT2, WRIT4, WRIT5... SUBROUTINE PACKAGE 'WRIT'.
C     UNIT... THIS PACKAGE.
C NOTE THAT THE ABOVE SUBROUTINES REFERENCE MANY OTHER EXTERNAL
C PROCEDURES FROM VARIOUS SUBROUTINE PACKAGES.  THESE INDIRECTLY
C REFERENCED PROCEDURES ARE NOT NAMED HERE, BUT ARE LISTED IN THE
C PARTICULAR SUBROUTINE PACKAGES.
C
C ERRORS:
C     102...  LESS THAN 4 LOGICAL UNITS:
C             FOUR LOGICAL UNITS MUST BE AVAILABLE TO READ THE INPUT
C             DATA AND TO WRITE THE OUTPUT LOG FILE.
C
C DATE: 1990, JANUARY 22
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     STORAGE LOCATIONS:
C
C     AUXILIARY STORAGE LOCATIONS:
      CHARACTER*80 TEXT
      INTEGER I
C
C     TEXTC...THE NAME OF THE DATA. STRING OF 80 CHARACTERS.
C     I...    AUXILIARY LOOP VARIABLE
C
C     QUANTITIES DESCRIBING ELEMENTARY WAVES AND THEIR RAYS:
      REAL PAR1,PAR2,YL(6),Y(35),YY(5)
      INTEGER IWAVE,IWAVE0,IKODE,IRAY,IY(12),IEND
C
C     IWAVE...INDEX OF THE COMPUTED ELEMENTARY WAVE.
C     IWAVE0..INDEX OF THE ALREADY COMPUTED ELEMENTARY WAVE HAVING THE
C             MOST NUMEROUS COMMON ELEMENTS WITH THE CURRENT ELEMENTARY
C             WAVE.
C     IKODE...THE LENGTH OF THE COMMON PART OF THE CODES OF THE IWAVE-TH
C             AND IWAVE0-TH ELEMENTARY WAVES.
C     IRAY... INDEX OF THE COMPUTED RAY.
C     PAR1,PAR2... RAY TAKE-OFF PARAMETERS.
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT A POINT OF A RAY (SEE
C             C.R.T.5.5.4).  THE QUANTITIES ARE LISTED IN THE SUBROUTINE
C             PACKAGE 'RAY'.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG A RAY
C             (SEE C.R.T.5.2.1).  THE QUANTITIES ARE LISTED IN THE
C             SUBROUTINE PACKAGE 'RAY'.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             A RAY (SEE C.R.T.5.2.2).  THE QUANTITIES ARE LISTED IN THE
C             SUBROUTINE PACKAGE 'RAY'.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG A RAY (SEE C.R.T.5.2.2).  THE QUANTITIES ARE LISTED
C             IN THE SUBROUTINE PACKAGE 'RAY'.
C     IEND... REASON OF THE TERMINATION OF THE COMPUTATION OF A RAY (SEE
C             C.R.T.5.4).
C
C     QUANTITIES DESCRIBING DATA FILES AND LOGICAL UNITS:
      INTEGER IFILE,NFILE,IU,NU,LUCODE,LURPAR,LUWRIT
      PARAMETER (NFILE=8)
      CHARACTER*80 FILE(NFILE)
      PARAMETER (NU=4)
      INTEGER LU(NU),KU(NU)
      DATA LU/5,4,3,2/
C
C     IFILE, NFILE, FILE, IU, NU, LU, KU... FOR THE DESCRIPTION OF THESE
C             QUANTITIES SEE THE SUBROUTINE UNIT BELOW.
C     LUCODE..THE LOGICAL UNIT CONNECTED TO THE FILE WITH THE CODE DATA.
C     LURPAR..THE LOGICAL UNIT CONNECTED TO THE FILE WITH THE RPAR DATA.
C     LUWRIT..THE LOGICAL UNIT CONNECTED TO THE FILE WITH THE WRIT DATA.
C
C.......................................................................
C
C     OPENING DATA FILES AND READING THE INPUT DATA:
C
C     THE NAME OF THE MAIN INPUT FILE.  THIS FILE CONTAINS THE NAMES OF
C     OTHER INPUT FILES
      FILE(1)=' '
      IF(NU.LT.4) THEN
        STOP 'ERROR 102 IN CRT(MAIN): LESS THAN 4 LOGICAL UNITS'
      END IF
C
C     (1) MAIN DATA FILE - CONTAINS THE NAMES OF OTHER INPUT FILES
      CALL UNIT(1,NFILE,FILE,IU,NU,LU,KU)
      READ(LU(IU),*) TEXT
      DO 10 I=2,NFILE
        READ(LU(IU),*) FILE(I)
   10 CONTINUE
C
C     (2) DATA FOR MODEL
      CALL UNIT(2,NFILE,FILE,IU,NU,LU,KU)
      CALL MODEL1(LU(IU))
C
C     (3) DATA FOR COMPLETE RAY TRACING
      CALL UNIT(3,NFILE,FILE,IU,NU,LU,KU)
      CALL RAY1(LU(IU))
C
C     (4) DATA FOR INITIAL POINTS OF RAYS
      CALL UNIT(4,NFILE,FILE,IU,NU,LU,KU)
      CALL INIT1(LU(IU))
C
C     (5) FILE CONTAINING THE CODES OF ELEMENTARY WAVES
      CALL UNIT(5,NFILE,FILE,IU,NU,LU,KU)
C     THE DATA FILE FOR THE SUBROUTINE CODE1 REMAINS OPEN
      LUCODE=LU(IU)
      IU=0
C
C     (6) FILE CONTROLING THE TAKE-OFF PARAMETERS OF RAYS
      CALL UNIT(6,NFILE,FILE,IU,NU,LU,KU)
C     THE DATA FILE FOR THE SUBROUTINE RPAR1 REMAINS OPEN
      LURPAR=LU(IU)
      IU=0
C
C     (7) FILE SPECIFYING THE NAMES OF FILES WITH THE COMPUTED
C     QUANTITIES
      CALL UNIT(7,NFILE,FILE,IU,NU,LU,KU)
C     THE DATA FILE FOR THE SUBROUTINE WRIT1 REMAINS OPEN
      LUWRIT=LU(IU)
      IU=0
C
C     (8) THE OUTPUT LOG FILE
      CALL UNIT(8,NFILE,FILE,IU,NU,LU,KU)
C     THE OUTPUT FILE FOR THE SUBROUTINES WRIT1, WRIT2, WRIT4 AND WRIT5
C     REMAINS OPEN
C
C.......................................................................
C
C     COMPLETE RAY TRACING:
C
      CALL WRIT1(LUWRIT,LU(IU),0,0,0)
      IWAVE=0
      IRAY=0
C
C     LOOP OVER ELEMENTARY WAVES
   30 CONTINUE
C       COMPUTATION OF A SINGLE ELEMENTARY WAVE:
C       READING THE INPUT DATA FOR THE ELEMENTARY WAVE
        CALL CODE1(LUCODE,IWAVE,IWAVE0,IKODE)
        IF(IWAVE.EQ.0) THEN
C         ALL REQUIRED ELEMENTARY WAVES ARE COMPUTED
          GO TO 90
        END IF
        CALL RPAR1(LURPAR,IWAVE)
        CALL WRIT1(LUWRIT,LU(IU),IWAVE,IWAVE0,IKODE)
C       LOOP OVER RAYS
   40   CONTINUE
C         COMPLETE TRACING OF A SINGLE RAY:
C         DETERMINATION OF THE TAKE-OFF PARAMETERS
          CALL RPAR2(IRAY,PAR1,PAR2)
          IF(IRAY.EQ.0) THEN
C           ALL REQUIRED RAYS OF THE ELEMENTARY WAVE ARE COMPUTED
            GO TO 80
          END IF
C         INITIAL CONDITIONS FOR THE RAY
          CALL INIT2(PAR1,PAR2,YL,Y,YY,IY,IEND,IWAVE0,IKODE)
          CALL WRIT2(LU(IU),IRAY)
          IF(IEND.EQ.0) THEN
C           COMPUTATION OF THE RAY
            CALL RAY2(YL,Y,YY,IY,IEND)
          END IF
C         THE RAY IS COMPUTED
          CALL RPAR4(IRAY,PAR1,PAR2,YL,Y,YY,IY,IEND)
          CALL WRIT4(LU(IU),IRAY,YL,Y,YY,IY,IEND)
        GO TO 40
   80   CONTINUE
C       THE ELEMENTARY WAVE IS COMPUTED
        CALL WRIT5(LU(IU),IWAVE)
      GO TO 30
   90 CONTINUE
C
C     END OF COMPUTATION
      CALL WRIT5(LU(IU),0)
      STOP
      END
C
C=======================================================================
C
      SUBROUTINE UNIT(IFILE,NFILE,FILE,IU,NU,LU,KU)
      INTEGER IFILE,NFILE,IU,NU,LU(NU),KU(NU)
      CHARACTER*(*) FILE(NFILE)
C
C SUBROUTINE UNIT IS DESIGNED TO CONTROL CONNECTING AND DISCONNECTING
C THE DATA FILES TO LOGICAL UNITS, AND TO DETERMINE THE LOGICAL UNITS
C FROM WHICH THE GIVEN DATA SETS ARE TO BE READ.
C
C INPUT:
C     IFILE...INDEX OF THE DATA SET TO BE READ IN.  THE DATA SETS ARE
C             INDEXED BY INTEGERS FROM 1 TO NFILE.
C     NFILE...THE TOTAL NUMBER OF DATA SETS.
C     FILE... CHARACTER ARRAY CONTAINING THE NAMES OF THE FILES
C             CONTAINING INDIVIDUAL DATA SETS.  DIFFERENT DATA SETS MAY
C             BE STORED IN THE SAME FILE.  IF IFILE=1, ONLY FILE(1) HAS
C             TO BE DEFINED.
C     IU...   INDEX OF THE LOGICAL UNIT CONNECTED TO THE FILE CONTAINING
C             THE LAST READ DATA SET (I.E. THE LAST DATA SET WAS READ
C             FROM THE LOGICAL UNIT LU(IU) CONNECTED TO THE FILE
C             FILE(KU(IU)).  ZERO IF NO DATA ARE READ IN, OR IF THERE IS
C             NO DATA FILE TO CLOSE.  NEED NOT BE DEFINED IF IFILE=1.
C     NU...   THE MAXIMUM NUMBER OF AVAILABLE LOGICAL UNITS.
C     LU...   ARRAY CONTAINING REFERENCE NUMBERS OF LOGICAL UNITS.
C     KU...   AUXILIARY ARRAY OF THE DIMENSION AT LEAST NU.
C             ITS ELEMENTS KU(1) TO KU(NU) MUST NOT BE MODIFIED BETWEEN
C             TWO INVOCATIONS OF THIS SUBROUTINE.  ITS VALUES NEED NOT
C             BE DEFINED IF IFILE=1.
C             KU(I)...ZERO IF THE LOGICAL UNIT LU(I) IS CLOSED,
C               OTHERWISE THE SEQUENTIAL NUMBER OF THE NEXT DATA SET TO
C               BE READ FROM THIS UNIT.
C
C OUTPUT:
C     IU...   INDEX OF THE LOGICAL UNIT CONNECTED TO FILE WITH THE DATA
C             SET TO BE READ IN (I.E. THE NEXT DATA SET WILL BE READ
C             FROM THE LOGICAL UNIT LU(IU) CONNECTED TO THE FILE
C             FILE(IFILE)).  ZERO IF NO LOGICAL UNIT IS AVAILABLE.
C     KU...   UPDATED INPUT VALUES.
C
C ERRORS:
C     101...  OPEN FILE ERROR:
C             ERROR ENCOUNTERED DURING EXECUTION OF THE OPEN STATEMENT.
C
C DATE: 1990, FEBRUARY 6
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
      INTEGER IERR,JU,I
C
      IF(IFILE.EQ.1) THEN
C       NO LOGICAL UNITS ARE CONNECTED WHEN OPENING THE FIRST DATA FILE
        DO 10 JU=1,NU
          KU(JU)=0
   10   CONTINUE
      ELSE
C       UPDATING INDICES OF NEXT DATA SETS TO BE READ FROM OPEN FILES:
        DO 13 JU=1,NU
          IF(0.LT.KU(JU).AND.KU(JU).LT.IFILE) THEN
C           THE DATA SET FROM THE FILE CONNECTED TO THE LOGICAL UNIT
C           LU(JU) HAS BEEN READ.  DETERMINING THE NEXT DATA SET
C           CONTAINED IN THE FILE:
            DO 11 I=IFILE,NFILE
              IF(FILE(KU(JU)).EQ.FILE(I)) THEN
C               THE I-TH DATA SET WILL ALSO BE READ FROM THE LAST FILE
C               CONNECTED TO THE LOGICAL UNIT LU(JU)
                KU(JU)=I
                GO TO 12
              END IF
   11       CONTINUE
C           NO OTHER DATA SET WILL BE READ FROM THE LAST FILE.  THE FILE
C           MAY BE CLOSED AND THE LOGICAL UNIT LU(IU) DICONNECTED
   12       CONTINUE
          END IF
   13   CONTINUE
C       CLOSING THE DATA FILE:
        IF(IU.GT.0) THEN
C         THERE IS A FILE SUBMITTED TO BE CLOSED
          IF(0.LT.KU(IU).AND.KU(IU).LT.IFILE) THEN
C           NO OTHER DATA SET WILL BE READ FROM THIS FILE.  THE FILE
C           MAY BE CLOSED AND THE LOGICAL UNIT LU(IU) DICONNECTED
            CLOSE(LU(IU))
            KU(IU)=0
          END IF
        END IF
      END IF
C
C     OPENING NEW DATA FILE:
      IF(IFILE.GT.0) THEN
        DO 21 JU=1,NU
          IF(KU(JU).EQ.IFILE) THEN
C           THE DATA FILE IS ALREADY OPEN
            IU=JU
            RETURN
          END IF
   21   CONTINUE
C       THE DATA FILE HAS TO BE OPENED
        DO 22 JU=1,NU
          IF(KU(JU).EQ.0) THEN
C           THE LOGICAL UNIT LU(JU) MAY BE CONNECTED TO THE DATA FILE
            IU=JU
            KU(IU)=IFILE
            IF(FILE(IFILE).EQ.' ') THEN
              OPEN(LU(IU),FILE=' ',STATUS='OLD',IOSTAT=IERR,ERR=90)
            ELSE
              OPEN(LU(IU),FILE=FILE(IFILE),IOSTAT=IERR,ERR=90)
            END IF
            RETURN
          END IF
   22   CONTINUE
C       NO LOGICAL UNIT AVAILABLE
        IU=0
      END IF
      RETURN
C
   90 CONTINUE
      WRITE(*,'('' STATUS'',I5,'': '',A)') IERR,FILE(IFILE)
      STOP 'ERROR 101 IN UNIT: OPEN FILE ERROR'
      END
C
C======================================================================
C
