C GUIDE TO COMPLETE RAY TRACING
C
C AUTHORS:
C     VLASTISLAV CERVENY
C             MFF UK
C             KE KARLOVU 3
C             121 16  PRAHA 2,  CZECHOSLOVAKIA
C     LUDEK KLIMES
C             UGG CSAV, ODD. 204
C             PUSKINOVO NAM. 9
C             160 00  PRAHA 6,  CZECHOSLOVAKIA
C     IVAN PSENCIK
C             GFU CSAV
C             BOCNI II, 1401
C             141 31  PRAHA 4,  CZECHOSLOVAKIA
C
C REFERENCE:
C     CERVENY V., KLIMES L. AND PSENCIK I.: COMPLETE SEISMIC-RAY TRACING
C             IN THREE-DIMENSIONAL STRUCTURES.  IN: SEISMOLOGICAL
C             ALGORITHMS, ED. DOORNBOS D.J., ACADEMIC PRESS, NEW YORK
C             1988.
C     THE ABOVE PAPER CONTAINS DETAILED DESCRIPTION OF THE ALGORITHM OF
C     COMPLETE RAY TRACING AND IS FREQUENTLY REFERRED THROUGHOUT THE
C     CODE OF COMPLETE RAY TRACING.  THE REFERENCES LIKE (C.R.T.5.4) OR
C     (5.4) ARE RELATED TO THE PAPER.  THE DETAILED DESCRIPTION OF THE
C     INPUT DATA, SUBROUTINE AND FUNCTION PARAMETERS, AND OTHER
C     SPECIFICATIONS NOT CONTAINED IN THE PAPER, IS INCLUDED IN THE
C     INDIVIDUAL PROGRAM PACKAGES.
C
C THE CODE OF THE COMPLETE RAY TRACING CONSISTS OF THE FOLLOWING
C SUBROUTINE PACKAGES:
C     CRT...  MAIN PROGRAM.  IT READS THE INPUT DATA AND THEN CONTROLS
C             THE COMPLETE RAY TRACING OF THE SPECIFIED ELEMENTARY
C             WAVES.
C     METRIC..SUBROUTINE EVALUATING THE METRIC TENSOR AND CHRISTOFFEL
C             SYMBOLS AT A GIVEN POINT (SEE C.R.T.2), AND THE RELATED
C             SUBROUTINES AND EXTERNAL FUNCTIONS.
C     MODEL...SUBROUTINES READING INPUT DATA TO SPECIFY THE SEISMIC
C             MODEL FOR COMPLETE RAY TRACING (SEE C.R.T.3.2.1), AND
C             AUXILIARY SUBROUTINES RELATED TO THE MODEL (C.R.T.3.3).
C     SRFC... SUBROUTINES FOR SPECIFICATION AND INTERPOLATION, IN
C             RECTANGULAR GRIDS, OF THE FUNCTIONS DESCRIBING SMOOTH
C             SURFACES IN THE MODEL (SEE C.R.T.3.2.2).
C     PARM... SUBROUTINES FOR SPECIFICATION AND INTERPOLATION, IN
C             RECTANGULAR GRIDS, OF THE MATERIAL PARAMETERS WITHIN
C             INDIVIDUAL COMPLEX BLOCKS IN THE MODEL (SEE C.R.T.3.2.3).
C     VAL...  SUBROUTINES CONTROLING FUNCTION SPECIFICATION AND
C             INTERPOLATION.  DESIGNED TO PERFORM THE INTERPOLATION
C             OF A SET OF FUNCTIONS IN A RECTANGULAR GRID, EMPLOYING
C             SPLINES UNDER TENSION.  THESE SUBROUTINES ARE REFERENCED
C             BY THE SUBROUTINES OF PACKAGES 'SRFC', 'PARM' AND 'INIT'.
C     FITCRT..SOME ROUTINES TAKEN FROM CLINE'S SOFTWARE PACKAGE
C             'FITPACK', CALLED BY THE SUBROUTINES OF THE PACKAGE 'VAL'
C             TO PERFORM THE SPLINE-UNDER-TENSION INTERPOLATION.
C     CODE... SUBROUTINE PACKAGE DEVOTED TO THE CODES FOR ELEMENTARY
C             WAVES.  CONTAINS THE GENERAL DESCRIPTION OF THE CODES FOR
C             ELEMENTARY WAVES AND SUBROUTINES DESIGNED TO READ THE
C             INPUT DATA FOR THE CODES OF ELEMENTARY WAVES AND TO
C             TRANSFORM THE USED NUMERICAL CODE OF THE ELEMENTARY WAVE
C             UNDER CONSIDERATION INTO INSTRUCTIONS SPECIFYING THE
C             BEHAVIOUR OF THE WAVE AT THE INITIAL POINTS OF RAYS AND AT
C             ALL POINTS OF INCIDENCE OF THE RAYS AT INTERFACES (SEE
C             C.R.T.4).
C     RAY...  CONTAINS THE SUBROUTINE DESIGNED TO READ THE INPUT DATA
C             FOR COMPLETE RAY TRACING AND TO STORE THEM IN THE MEMORY
C             (SEE C.R.T.5.6), THE LIST OF THE QUANTITIES COMPUTED ALONG
C             RAYS (SEE C.R.T.5.2), AND THE SUBROUTINE DESIGNED TO
C             CONTINUE THE COMPLETE RAY TRACING OF A RAY FROM THE GIVEN
C             POINT (SEE C.R.T.5.7).
C     RAYCB...SUBROUTINES FOR COMPLETE RAY TRACING WITHIN ONE COMPLEX
C             BLOCK (SEE C.R.T.5.8).
C     HPCG... SUBROUTINE OF THE IBM SCIENTIFIC SUBROUTINE PACKAGE
C             SOLVING A SYSTEM OF GENERAL FIRST ORDER ORDINARY
C             DIFFERENTIAL EQUATIONS WITH GIVEN INITIAL VALUES.  IT IS
C             CALLED BY THE SUBROUTINE RAYCB OF THE PACKAGE 'RAYCB'.
C     TRANS...SUBROUTINE TRANSFORMING THE COMPUTED QUANTITIES ACROSS A
C             CURVED INTERFACE (SEE C.R.T.5.9), AND SUBROUTINE REPLACING
C             THE AMPLITUDES BY ONES INVOLVING THE APPROPRIATE
C             CONVERSION COEFFICIENTS (SEE C.R.T.5.5.4).
C     COEF... SUBROUTINES COMPUTING THE REFLECTION/TRANSMISSION
C             COEFFICIENTS (SEE C.R.T.5.9.7).
C     INIT... SUBROUTINES DESIGNED TO READ THE INPUT DATA FOR THE
C             INITIAL SURFACE, AND TO DEFINE THE INITIAL VALUES OF THE
C             QUANTITIES FOR COMPLETE TRACING OF A RAY WITH GIVEN
C             TAKE-OFF PARAMETERS (SEE C.R.T.6).
C     RPAR... SUBROUTINES CONTROLING THE TAKE-OFF PARAMETERS OF THE
C             RAYS.  A USER MAY INTRODUCE HIS OWN PROCEDURE OF SELECTION
C             OF THE TAKE-OFF PARAMETERS AND HIS OWN TWO-POINT RAY
C             TRACING ALGORITHM BY MEANS OF A MODIFICATION OF THIS
C             SUBROUTINE PACKAGE.
C     WRIT... SUBROUTINES CREATING THE OUTPUT OF COMPLETE RAY TRACING
C             (SEE C.R.T.5.5).
C     SCRONUL... DUMMY VERSION OF THE SUBROUTINE PACKAGE 'SCRO'
C             CONTAINING SCREEN OUTPUT SUBROUTINES CALLED BY THE
C             SUBROUTINES OF THE PACKAGE 'WRIT'.
C THE ABOVE PACKAGES FORM A CONSISTENT BASIC VERSION OF THE COMPLETE RAY
C TRACING PROGRAM.  NOTE THAT NOT ALL OF THE SUBROUTINES OF THE SOFTWARE
C PACKAGE 'FITCRT' ARE USED IN THE BASIC VERSION.  THE FOLLOWING
C SUBROUTINE PACKAGES MAY BE USED TO MODIFY THE BASIC VERSION OF THE
C COMPLETE RAY TRACING PROGRAM:
C     SCROPC..VERSION OF THE SUBROUTINE PACKAGE 'SCRO' FOR THE
C             IBM-COMPATIBLE PERSONAL COMPUTERS.  IT SHOULD ALSO WORK ON
C             THE VAX COMPUTERS.  THE SCREEN OUTPUT CONSISTS OF A SIMPLE
C             GRAPHIC OUTPUT (SEE THE SUBROUTINE PACKAGES 'PLOT' BELOW)
C             AND OF WRITING THE BRIEF MESSAGES ON THE STATUS OF THE
C             CURRENTLY COMPUTED RAY TO THE CONSOLE.  THIS OUTPUT
C             CONTROLS THE CONSOLE BY MEANS OF THE ANSI ESCAPE SEQUENCES
C             SUPPORTED BY MS-DOS ANSI.SYS DRIVER ON IBM-COMPATIBLE
C             PERSONAL COMPUTERS.  THIS CONSOLE OUTPUT IS JUST AN
C             EXAMPLE, THE SUBROUTINES SCRO1, SCRO2, SCRO3, SCRO4 AND
C             SCRO5 HAVE TO BE MODIFIED BY A USER FOR THE PARTICULAR
C             COMPUTER SYSTEM.  IF NO SCREEN OUTPUT IS REQUIRED, THE
C             EXECUTABLE STATEMENTS OF THE SUBROUTINES SCRO1, SCRO2,
C             SCRO3, SCRO4 AND SCRO5 MAY SIMPLY BE DELETED (SEE
C             'SCRONUL' ABOVE).
C     PLOTNUL... DUMMY VERSION OF THE SUBROUTINE PACKAGE 'PLOT'
C             CONTAINING THE PLOT SUBROUTINES CALLED BY THE SUBROUTINES
C             OF THE PACKAGE 'SCROPC', SEE ALSO THE SECTION GRAPHICS OF
C             THIS GUIDE.
C     PLOTVDI... VERSION OF THE SUBROUTINE PACKAGE 'PLOT' CONTAINING THE
C             PLOT INTERFACE SUBROUTINES BETWEEN THE SUBROUTINE PACKAGE
C             'SCROPC' AND THE GSS-VDI GRAPHIC FUNCTIONS.
C     PLOTH...VERSION OF THE SUBROUTINE PACKAGE 'PLOT' CONTAINING THE
C             PLOT INTERFACE SUBROUTINES BETWEEN THE SUBROUTINE PACKAGE
C             'SCROPC' AND THE HALO GRAPHIC FUNCTIONS.
C
C RELEASED VERSIONS:
C     1988, JUNE:  PRELIMINARY VERSION, CONTAINS ONLY SUBROUTINES FOR
C             MODEL SPECIFICATION, NO COMPLETE RAY TRACING SUBROUTINES.
C     1990, JANUARY:  FIRST CONSISTENT VERSION OF THE RAY TRACING
C             PROGRAM.  THE ROUTINES DETERMINING THE TAKE-OFF PARAMETERS
C             OF RAYS ARE AWFULLY PRIMITIVE.  NOT WELL-DEBUGGED.
C
C OUTPUT STATEMENTS:
C     ERROR MESSAGES:
C             THE ERROR MESSAGES ARE MOST FREQUENTLY WRITTEN BY
C             STATEMENTS LIKE
C               STOP 'ERROR MESSAGE',
C             SOMETIMES LIKE
C               WRITE(*,'('' ERROR MESSAGE'')').
C             THEY ARE SPREAD OVER ALL SUPROUTINE PACKAGES.  THEY ARE
C             LISTED IN THE DESCRIPTIONS OF INDIVIDUAL SUBROUTINES.  THE
C             FIRST ONE OR TWO DIGITS OF THE ERROR CODE ROUGHLY INDICATE
C             THE SUBROUTINE PACKAGE:
C               1**... 'CRT',
C               2**... 'METRIC',
C               3**... 'MODEL', 'SRFC', 'PARM', 'VAL',
C               31*... 'MODEL',
C               32*... 'PARM',
C               35*... 'VAL',
C               4**... 'CODE',
C               55*... 'WRIT',
C               56*... 'RAY',
C               57*... 'RAY',
C               58*... 'RAYCB',
C               59*... 'TRANS', 'COEF',
C               6**... 'INIT',
C               7**... 'RPAR'.
C     OTHER OUTPUTS:
C             ARE CONCENTRATED IN THE SUBROUTINE PACKAGE 'WRIT'.
C     SCREEN OUTPUT:
C             IS CONCENTRATED IN THE SUBROUTINE PACKAGE 'SCRO', IN THE
C             SUBROUTINES SCRO1, SCRO2, SCRO3, SCRO4 AND SCRO5.
C
C DATA SETS:
C     THE COMPLETE RAY TRACING PROGRAM READS OR MAY WRITE THE FOLLOWING
C     DATA SETS:
C (1) CRT...  MAIN DATA SET WITH THE NAMES OF THE FILES CONTAINING THE
C             DATA SETS (2) TO (8).  IT IS READ IN BY THE MAIN PROGRAM
C             CRT AND IS DESCRIBED IN THE PACKAGE 'CRT'.
C (2) MODEL...INPUT DATA FOR THE MODEL.
C             THE DATA ARE READ BY THE SUBROUTINE MODEL1 AND ARE
C             DESCRIBED IN THE SUBROUTINE PACKAGE 'MODEL'.
C (3) DCRT... INPUT DATA FOR THE COMPLETE RAY TRACING.
C             THE DATA ARE READ BY THE SUBROUTINE RAY1 AND ARE
C             DESCRIBED IN THE SUBROUTINE PACKAGE 'RAY'.
C (4) INIT... INPUT DATA SPECIFYING THE INITIAL CONDITIONS FOR RAYS.
C             THE DATA ARE READ BY THE SUBROUTINE INIT1 AND ARE
C             DESCRIBED IN THE SUBROUTINE PACKAGE 'INIT'.
C (5) CODE... INPUT DATA SPECIFYING THE CODES OF ELEMENTARY WAVES.
C             THE DATA ARE READ BY THE SUBROUTINE CODE1 AND ARE
C             DESCRIBED IN THE SUBROUTINE PACKAGE 'CODE'.
C (6) RPAR... INPUT DATA SPECIFYING THE TAKE-OFF PARAMETERS OF THE
C             COMPUTED RAYS.  THE DATA ARE READ BY THE SUBROUTINE
C             RPAR1 AND ARE DESCRIBED IN THE SUBROUTINE PACKAGE 'RPAR'.
C (7) WRIT... NAMES OF THE FILES CONTAINING THE DATA SETS (9) TO (11).
C             THIS DATA SET IS READ BY THE SUBROUTINE WRIT1 AND IS
C             DESCRIBED IN THE SUBROUTINE PACKAGE 'WRIT'.
C (8) LOG...  LOG OUTPUT.
C             THE DATA ARE WRITTEN BY THE SUBROUTINES WRIT1, WRIT2,
C             WRIT4 AND WRIT5 AND ARE DESCRIBED IN THE SUBROUTINE
C             PACKAGE 'WRIT'.
C (9) (WRIT31(I),I=1,KWAVE)... OUTPUT DATA SETS WITH THE COMPUTED
C             QUANTITIES ALONG THE RAYS (SEE C.R.T.5.5.3).  KWAVE IS THE
C             NUMBER OF THE COMPUTED ELEMENTARY WAVES, OR KWAVE=1 IF THE
C             QUANTITIES CORRESPONDING TO ALL WAVES ARE STORED INTO A
C             SINGLE FILE.  THE DATA ARE WRITTEN ESPECIALLY BY THE
C             SUBROUTINE WRIT31 AND ARE DESCRIBED IN THE SUBROUTINE
C             PACKAGE 'WRIT'.  THE THE DATA SETS MAY ALSO BE, IN PART,
C             WRITTEN BY THE SUBROUTINES WRIT1, WRIT2, WRIT4 OR WRIT5.
C             UNFORMATTED.
C (10) (WRIT32(I),I=1,NSTOR)... OUTPUT DATA SETS WITH THE COMPUTED
C             QUANTITIES AT THE SPECIFIED SURFACES (SEE C.R.T.5.5.2).
C             NSTOR IS THE NUMBER OF SURFACES AT WHICH THE QUANTITIES
C             ARE TO BE STORED.  THE DATA ARE WRITTEN ESPECIALLY BY THE
C             SUBROUTINE WRIT32 AND ARE DESCRIBED IN THE SUBROUTINE
C             PACKAGE 'WRIT'.  THE THE DATA SETS MAY ALSO BE, IN PART,
C             WRITTEN BY THE SUBROUTINES WRIT1, WRIT2, WRIT4 OR WRIT5.
C             UNFORMATTED.
C (11) ((WRIT33(I,J),I=1,NELEM(J)),J=1,NWAVE)... OUTPUT DATA SETS WITH
C             THE COMPUTED QUANTITIES AT THE ENDPOINTS OF THE ELEMENTS
C             OF RAYS OF INDIVIDUAL ELEMENTARY WAVES (SEE C.R.T.5.5.3).
C             NWAVE IS THE NUMBER OF THE COMPUTED ELEMENTARY WAVES.
C             NELEM(J) IS THE NUMBER OF ELEMENTS OF RAYS OF J-TH
C             ELEMENTARY WAVE WHICH DO NOT COINCIDE WITH THE ELEMENTARY
C             WAVES WHICH HAVE BEEN COMPUTED.  THE DATA ARE WRITTEN
C             ESPECIALLY BY THE SUBROUTINE WRIT33 AND ARE DESCRIBED IN
C             THE SUBROUTINE PACKAGE 'WRIT'.  THE DATA SETS MAY ALSO BE,
C             IN PART, WRITTEN BY THE SUBROUTINES WRIT1, WRIT2, WRIT4 OR
C             WRIT5.  THESE FILES ARE ARE AUXILIARY DISK STORAGE
C             LOCATIONS TO THE PROGRAM CRT AND ARE NOT INTENDED TO BE
C             THE OUTPUT OF THE COMPLETE RAY TRACING USED BY THE USER
C             APPLICATION PROGRAMS.  UNFORMATTED.
C (12) FILE WITH THE QUANTITIES AT THE INITIAL POINTS OF RAYS (SEE
C             C.R.T.6.1) CORRESPONDING TO EACH OF THE FILES (9), (10)
C             AND (11).  UNFORMATTED.
C     SEVERAL DATA SETS MAY BE READ FROM OR WRITTEN TO A SINGLE FILE.
C     THE DATA SETS ARE READ IN THE FOLLOWING ORDER:
C             SUCCESIVELY READING DATA (1), (2), (3), (4) AND THE
C             INTRODUCTION PART OF (7).
C             WRITING THE INTRODUCTION PARTS OF (8) AND, PERHAPS, OF (9)
C             AND (10).
C             FOR EACH ELEMENTARY WAVE IWAVE: SUCCESIVELY READING THE
C             PARTS OF DATA (5), (6), (7) CORRESPONDING TO THIS
C             ELEMENTARY WAVE, THEN SIMULTANEOUSLY WRITING THE PARTS OF
C             (8), (9), (10), (11) CORRESPONDING TO THIS ELEMENTARY
C             WAVE.
C             WRITING THE END PARTS OF (8) AND, PERHAPS, OF (9) AND
C             (10).
C
C REASONS OF THE TERMINATION OF THE COMPUTATION OF A RAY
C     ARE SPECIFIED BY THE OUTPUT PARAMETER IEND OF THE SUBROUTINES
C     INIT2 AND RAY2 AND ARE LISTED IN THE SUBROUTINE PACKAGES 'INIT'
C     AND 'RAY'.
C
C LIST OF THE COMPUTED QUANTITIES IS GIVEN IN THE PACKAGE 'RAY'.
C
C PROGRAMMING LANGUAGE AND CODE LAYOUT:
C     PROGRAMMING LANGUAGE:
C             FORTRAN77-FULL LANGUAGE AS SPECIFIED IN THE ANSI STANDARD:
C             X3.9-1978: AMERICAN NATIONAL STANDARD PROGRAMMING LANGUAGE
C             FORTRAN.  AMERICAN NATIONAL STANDARDS INSTITUTE, NEW YORK
C             1978.
C     CHARACTER SET:
C             CHARACTER SET OF FORTRAN77 WITH CAPITAL LETTERS (49
C             CHARACTERS INCLUDING THE SPACE):
C             ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789 =+-*/(),.$':
C     LINE LENGTH:
C             72 CHARACTERS
C     INDENTATION:
C             DOCUMENTATION AND EXTENSIVE COMMENTS AT THE BEGINNINGS OF
C             SUBROUTINES:
C               COLUMNS 3, 7, 15, 17, 19, ETC. WITH STEP 2.
C             FORTRAN STATEMENTS AND COMMENTS WITHIN STATEMENTS:
C               COLUMNS 7, 9, 11, ETC. WITH STEP 2.
C             INSIDES OF DO LOOPS AND IF STATEMENTS ARE INDENTED.
C             INDENTATION OF THE CONTINUATION LINES IS HAPHAZARD IN THIS
C               VERSION.
C     EXAMPLE FORMS OF THE FORTRAN STATEMENTS AND OF THE SPACING:
C             A=B+5.*(E+F)
C               NO SPACES IN ASSIGNMENT STATEMENTS AND EXPRESSIONS,
C               EXCEPT WHEN MATCHING SIMILAR EXPRESSIONS IN SUCCEEDING
C               LINES.
C             GO TO 12
C             GO TO (10,20,30)K
C             IF(I.EQ.J) GO TO 30
C             IF(A.GT.B) THEN
C             ELSE IF(C.GT.D) THEN
C             END IF
C             DO 20 I=1,15
C             BLOCK DATA
C             BACKSPACE
C             CALL VEL(N,COOR,F)
C             SUBROUTINE VEL(N,COOR,F)
C             PARAMETER (PI=3.14159)
C             COMMON/ABC/IR,FF,HH
C             COMMON IR,FF,HH
C             DATA R/4.,2.,1./,K/0/
C             READ(LUN,*) L,M,(BER(I),I=LS,MS),ALFA,BETA
C               BUT:
C               READ (LUN,*) L,M,(BER(I),I=LS,MS),ALFA,BETA
C               WRITE(LUN,*) L,M,(BER(I),I=LS,MS),ALFA,BETA
C     MISCELANEOUS:
C             ALL SENTENCES IN DOCUMENTATION SHOULD BE ENDED BY A DOT.
C             INDIVIDUAL SECTIONS OF THE CODE MAY BE SEPARATED BY BLANK
C               LINES WITH C IN THE FIRST COLUMN.
C             ONLY CONTINUE AND FORMAT STATEMENTS SHOULD BE LABELED.
C             SPECIFIC NAMES OF THE INTRINSIC FUNCTIONS ARE PREFERRED TO
C               GENERIC ONES.
C
C GRAPHICS:
C     ALL GRAPHIC OUTPUTS ARE CONCENTRATED IN THE SUBROUTINE PACKAGE
C     'SCROPC',  IN SUBROUTINES SCRO1, SCRO2, SCRO3, SCRO4 AND SCRO5.
C     CALCOMP SUBROUTINES
C             PLOTS(0,0,0)
C             PLOT(XPAGE,YPAGE,IPEN)
C             NEWPEN(INP)
C     ARE CALLED.  THE SUBROUTINE SYMBOL CANNOT BE USED BECAUSE ITS
C     CALCOMP SPECIFICATION IS NOT CONFORMAL WITH THE FORTRAN77
C     STANDARD.  THE SUBROUTINE NUMBER IS NOT EMPLOYED.  IF THE
C     SUBROUTINES PLOTS, PLOT AND NEWPEN ARE NOT PRESENT IN THE
C     COMPUTER, THEY MUST BE SUBSTITUTED BY USER-DEFINED OR VOID
C     SUBROUTINES.  THE SPECIFICATION OF THEIR PARAMETERS CAN BE FOUND
C     IN THE SUBROUTINE PACKAGE 'SCROPC'.
C
C ACKNOWLEDGEMENTS:
C     THE DEVELOPEMENT OF THE COMPLETE RAY TRACING ALGORITHM HAS BEEN
C     PARTIALLY SUPPORTED BY:
C     FACULTY OF MATHEMATICS AND PHYSICS, CHARLES UNIVERSITY, PRAGUE.
C     INSTITUTE OF GEOLOGY AND GEOTECHNICS, CZECHOSLOVAK ACADEMY OF
C             SCIENCES, PRAGUE.
C     ENERGY, MINES AND RESOURCES CANADA, OTTAWA, ONTARIO.
C     GEOPHYSICAL INSTITUTE, CZECHOSLOVAK ACADEMY OF SCIENCES, PRAGUE.
C
C DATE: 1990, JANUARY 24
C
C=======================================================================
C
