C SUBROUTINE PACKAGE 'RPAR' TO CONTROL THE TAKE-OFF PARAMETERS OF RAYS.
C
C BY VLASTISLAV CERVENY, LUDEK KLIMES, IVAN PSENCIK
C
C THIS PACKAGE CONSISTS OF THE FOLLOWING SUBROUTINES:
C     RPARB...BLOCK DATA SUBROUTINE DEFINING COMMON BLOCK /RPARC/ TO
C             TO STORE THE INPUT DATA FOR THE TAKE-OFF PARAMETERS OF
C             RAYS.
C     RPAR1...SUBROUTINE DESIGNED TO READ THE INPUT DATA FOR THE
C             TAKE-OFF PARAMETERS , TO PREPARE THE PARAMETERS FOR THE
C             SUBROUTINE RPAR2 AND TO STORE THEM IN THE COMMON BLOCK
C             /RPARC/.  IT IS CALLED WHEN STARTING THE COMPUTATION OF A
C             NEW ELEMENTARY WAVE.
C     RPAR2...SUBROUTINE DESIGNED TO SPECIFY THE TAKE-OFF PARAMETERS OF
C             THE INDIVIDUAL RAYS.
C     RPAR31..SUBROUTINE CALLED WITH CONSTANT STEP STORE OF THE
C             INDEPENDENT VARIABLE ALONG THE RAY, AND AT THE POINTS OF
C             INTERSECTION WITH INTERFACES EITHER BEFORE AND AFTER THE
C             TRANSFORMATION, I.E. CALLED SIMULTANEOUSLY WITH THE
C             SUBROUTINE WRIT31 (SEE ALSO C.R.T.5.5.1).  IT MAY BE USED
C             TO KEEP THE QUANTITIES USEFUL FOR THE DETERMINATION OF THE
C             TAKE-OFF PARAMETERS IN THE MEMORY.
C     RPAR32..SUBROUTINE CALLED AT THE POINTS OF INTERSECTION OF THE RAY
C             WITH SPECIFIED INTERFACES, I.E. CALLED SIMULTANEOUSLY WITH
C             THE SUBROUTINE WRIT32 (SEE ALSO C.R.T.5.5.2).  IT MAY BE
C             USED TO KEEP THE QUANTITIES USEFUL FOR THE DETERMINATION
C             OF THE TAKE-OFF PARAMETERS IN THE MEMORY.
C     RPAR33..SUBROUTINE CALLED AT THE ENDPOINTS OF THE ELEMENTS OF
C             RAYS, SITUATED AT STRUCTURAL INTERFACES, I.E. CALLED
C             SIMULTANEOUSLY WITH THE SUBROUTINE WRIT33 (SEE ALSO
C             C.R.T.5.5.3).  IT MAY BE USED TO KEEP THE QUANTITIES
C             USEFUL FOR THE DETERMINATION OF THE TAKE-OFF PARAMETERS IN
C             THE MEMORY.
C     RPAR4...SUBROUTINE CALLED AFTER FINISHING THE COMPUTATION OF EACH
C             RAY.  IT DEFINES FOUR STORAGE LOCATIONS OF THE COMMON
C             BLOCK /INITC/ INTRODUCED IN THE PACKAGE 'INIT'.
C
C INPUT DATA FOR THE TAKE-OFF PARAMETERS OF RAYS:
C     THE DATA ARE READ IN BY THE LIST DIRECTED INPUT (FREE FORMAT).  IN
C     THE LIST OF INPUT DATA BELOW, EACH NUMBERED PARAGRAPH INDICATES
C     THE BEGINNING OF A NEW INPUT OPERATION (NEW READ STATEMENT).  IF
C     THE FIRST LETTER OF THE SYMBOLIC NAME OF THE INPUT VARIABLE IS
C     I-N, THE CORRESPONDING VALUE IN INPUT DATA MUST BE OF THE TYPE
C     INTEGER.  OTHERWISE (EXCEPT TEXTP), THE INPUT PARAMETER IS OF THE
C     TYPE REAL.
C (1) TEXTP
C     STRING DESCRIBING THE DATA.  ONLY THE FIRST 80 CHARACTERS OF THE
C     STRING ARE SIGNIFICANT.
C FOR EACH ELEMENTARY WAVE IWAVE THE FOLLOWING DATA (2) AND (3):
C (2) ANY TIMES:
C     PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
C     THE FIRST RAY TAKE-OFF PARAMETER PAR1 TAKES THE VALUES FROM PAR1L
C     TO PAR1M WITH A FIXED STEP OF THE SIZE ABS(PAR1D).  FOR PAR1D=0, A
C     SINGLE VALUE PAR1=PAR1L OF THE FIRST RAY PARAMETER IS CONSIDERED.
C     THE SECOND RAY TAKE-OFF PARAMETER PAR2 TAKES THE VALUES FROM PAR2L
C     TO PAR2M WITH A FIXED STEP OF THE SIZE ABS(PAR2D).  FOR PAR2D=0,
C     A SINGLE VALUE PAR1=PAR1L OF THE SECOND RAY PARAMETER IS
C     CONSIDERED.
C     DEFAULT VALUES: ZEROES WHEN STARTING THE COMPLETE RAY TRACING
C     PROGRAM, OTHERWISE THE PREVIOUS VALUES.
C (3) /(A SLASH)
C
C STORAGE IN THE MEMORY:
C     THE INPUT DATA (2) ARE STORED IN COMMON BLOCK /RPARC/ DEFINED IN
C     THE FOLLOWING SUBROUTINE:
C     ------------------------------------------------------------------
      BLOCK DATA RPARB
      INTEGER LURPAR
      REAL PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
      COMMON/RPARC/ LURPAR,PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
      SAVE /RPARC/
      END
C     ------------------------------------------------------------------
C     LURPAR..LOGICAL UNIT NUMBER OF THE EXTERNAL INPUT DEVICE
C             CONTAINING THE INPUT DATA.
C     PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D... LAST VALUES OF THE INPUT
C             DATA (2).
C
C DATE: 1990, JANUARY 24
C CODED BY LUDEK KLIMES
C
C=======================================================================
C
      SUBROUTINE RPAR1(LUN,IWAVE)
      INTEGER LUN,IWAVE
C
C THIS SUBROUTINE IS CALLED WHEN STARTING THE COMPUTATION OF A NEW
C ELEMENTARY WAVE.  IT PREPARES THE PARAMETERS FOR THE SUBROTINE RPAR2
C TO SPECIFY THE TAKE-OFF PARAMETERS OF THE INDIVIDUAL RAYS.
C
C INPUT:
C     LUN...  LOGICAL UNIT NUMBER OF THE EXTERNAL INPUT DEVICE
C             CONTAINING THE INPUT DATA.
C     IWAVE...INDEX OF THE ELEMENTARY WAVE THAT IS GOING TO BE COMPUTED,
C             I.E. THE OUTPUT FROM THE LAST INVOCATION OF THE SUBROUTINE
C             CODE1.
C
C NO OUTPUT.
C
C COMMON BLOCK /RPARC/:
      INTEGER LURPAR
      REAL PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
      COMMON/RPARC/ LURPAR,PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
C ALL THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE DEFINED IN THIS
C SUBROUTINE.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1990, JANUARY 19
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
      CHARACTER*80 TEXTP
C
C     TEXTP...THE NAME OF THE DATA. STRING OF 80 CHARACTERS.
C
      IF(IWAVE.EQ.1) THEN
C       READING THE NAME OF THE INPUT DATA
        READ(LUN,*) TEXTP
C
C       DEFAULT VALUES
        PAR1L=0.
        PAR1M=0.
        PAR1D=0.
        PAR2L=0.
        PAR2M=0.
        PAR2D=0.
      END IF
C
      LURPAR=LUN
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE RPAR2(IRAY,PAR1,PAR2)
      REAL PAR1,PAR2
      INTEGER IRAY
C
C THIS SUBROUTINE DETERMINES THE TAKE-OFF PARAMETERS OF THE RAY.
C
C INPUT:
C     IRAY... NUMBER OF THE ALREADY COMPUTED RAYS.  IRAY=0 WHEN
C             BEGINNING THE COMPUTATION OF A NEW ELEMENTARY WAVE.
C             OTHERWISE, THE OUTPUT FROM THE PREVIOUS INVOCATION OF
C             RPAR2.
C
C OUTPUT:
C     IRAY... IRAY=0 IF ALL RAYS ARE COMPUTED AND THE COMPUTATION OF THE
C             ELEMENTARY WAVE WILL BE TERMINATED.
C             OTHERWISE, INPUT VALUE INCREASED BY 1.
C     PAR1,PAR2... TAKE-OFF PARAMETERS OF THE RAY.
C NONE OF THE INPUT PARAMETERS EXCEPT IRAY ARE ALTERED.
C
C COMMON BLOCK /RPARC/:
      INTEGER LURPAR
      REAL PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
      COMMON/RPARC/ LURPAR,PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
C ALL THE STORAGE LOCATIONS, EXCEPT LURPAR, OF THE COMMON BLOCK ARE
C REDEFINED IN THIS SUBROUTINE.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1989, DECEMBER 21
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
C     REAL AUX1,AUX2,AUX3,AUX4,AUX5,AUX6
C
      IRAY=IRAY+1
      IF(IRAY.EQ.1) THEN
        GO TO 10
      END IF
      PAR1=PAR1+PAR1D
      IF(PAR1D*(PAR1-PAR1M-PAR1D/1000.).GE.0.) THEN
        PAR1=PAR1L
        PAR2=PAR2+PAR2D
        IF(PAR2D*(PAR2-PAR2M-PAR2D/1000.).GE.0.) THEN
          GO TO 10
        END IF
      END IF
      RETURN
C
C     READING INPUT DATA:
   10 CONTINUE
      AUX1=PAR1L
      AUX2=PAR1M
      AUX3=PAR1D
      AUX4=PAR2L
      AUX5=PAR2M
      AUX6=PAR2D
      READ(LURPAR,*) PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
      IF(IRAY.NE.1) THEN
        IF(PAR1L.EQ.AUX1.AND.PAR1M.EQ.AUX2.AND.PAR1D.EQ.AUX3.AND.
     *     PAR2L.EQ.AUX4.AND.PAR2M.EQ.AUX5.AND.PAR2D.EQ.AUX6) THEN
          IRAY=0
          RETURN
        END IF
      END IF
      IF((PAR1M-PAR1L)*PAR1D.LT.0.) THEN
        PAR1D=-PAR1D
      END IF
      IF((PAR2M-PAR2L)*PAR2D.LT.0.) THEN
        PAR2D=-PAR2D
      END IF
      PAR1=PAR1L
      PAR2=PAR2L
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE RPAR31(YL,Y,YY,IY)
      REAL YL(6),Y(35),YY(5)
      INTEGER IY(12)
C
C THIS SUBROUTINE MAY BE USED TO STORE THE QUANTITIES USEFUL FOR THE
C DETERMINATION OF THE TAKE-OFF PARAMETERS IN THE MEMORY.  IT IS CALLED
C WITH CONSTANT STEP STORE (SEE THE INPUT DATA IN THE SUBROUTINE PACKAGE
C 'RAY') OF THE INDEPENDENT VARIABLE ALONG THE RAY, AND AT THE POINTS OF
C INTERSECTION WITH INTERFACES EITHER BEFORE AND AFTER THE
C TRANSFORMATION.
C
C INPUT:
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.
C NONE OF THE INPUT PARAMETERS ARE ALTERED.
C
C NO OUTPUT.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1990, JANUARY 15
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE RPAR32(ISTOR,YL,Y,YY,IY,NAMPL,YC)
      INTEGER ISTOR,IY(12),NAMPL
      REAL YL(6),Y(27),YY(5),YC(NAMPL)
C
C THIS SUBROUTINE MAY BE USED TO STORE THE QUANTITIES USEFUL FOR THE
C DETERMINATION OF THE TAKE-OFF PARAMETERS IN THE MEMORY.  IT IS CALLED
C AT THE POINTS OF INTERSECTION OF THE RAY WITH INTERFACES SPECIFIED IN
C THE INPUT DATA IN THE SUBROUTINE PACKAGE 'RAY' FOR STORING THE
C COMPUTED QUANTITIES (SEE ALSO C.R.T.5.5.2).
C
C INPUT:
C     ISTOR...THE SEQUENTIAL NUMBER IN THE IPUT DATA OF THE SPECIFIED
C             SURFACE.
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C             QUANTITIES Y(28) TO Y(IY(1)) REPRESENTING REDUCED
C             AMPLITUDES ARE IGNORED.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.
C     NAMPL...NUMBER OF REAL QUANTITIES REPRESENTING COMPLEX-VALUED
C             VECTORIAL REDUCED AMPLITUDES.  IF NO CONVERSION
C             COEFFICIENTS ARE APPLIED NAMPL=IY(1)-27,  OTHERWISE
C             NAMPL=6 OR 12 (SEE C.R.T.5.5.4).
C     YC...   ARRAY CONTAINING REAL QUANTITIES REPRESENTING COMPLEX-
C             -VALUED VECTORIAL REDUCED AMPLITUDES.  IF NO CONVERSION
C             COEFFICIENTS ARE APPLIED, YC IS A COPY OF Y(28) TO
C             Y(IY(1)).  OTHERWISE,  YC REPRESENTS THE VECTORIAL REDUCED
C             AMPLITUDES INVOLVING APPROPRIATE CONVERSION COEFFICIENTS,
C             EXPRESSED IN RAY-CENTRED COORDINATE SYSTEM (SEE
C             C.R.T.5.5.4):
C             P WAVE AT THE INITIAL POINT OF THE RAY:
C               NAMPL=6,
C               YC(1)=REAL(A13),  YC(2)=AIMAG(A13),
C               YC(3)=REAL(A23),  YC(4)=AIMAG(A23),
C               YC(5)=REAL(A33),  YC(6)=AIMAG(A33).
C             S WAVE AT THE INITIAL POINT OF THE RAY:
C               NAMPL=12,
C               YC(1)=REAL(A11),  YC(2)=AIMAG(A11),
C               YC(3)=REAL(A21),  YC(4)=AIMAG(A21),
C               YC(5)=REAL(A31),  YC(6)=AIMAG(A31),
C               YC(7)=REAL(A12),  YC(8)=AIMAG(A12),
C               YC(9)=REAL(A22),  YC(10)=AIMAG(A22),
C               YC(11)=REAL(A32), YC(12)=AIMAG(A32).
C NONE OF THE INPUT PARAMETERS ARE ALTERED.
C
C NO OUTPUT.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1990, JANUARY 15
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE RPAR33(YL,Y,YY,IY)
      REAL YL(6),Y(35),YY(5)
      INTEGER IY(12)
C
C THIS SUBROUTINE MAY BE USED TO STORE THE QUANTITIES USEFUL FOR THE
C DETERMINATION OF THE TAKE-OFF PARAMETERS IN THE MEMORY.  IT IS CALLED
C AT THE ENDPOINTS OF THE ELEMENTS OF RAYS, SITUATED AT STRUCTURAL
C INTERFACES  (SEE ALSO C.R.T.5.5.3).
C
C INPUT:
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.  FOR THE DEFINITION OF ARRAYS YL, Y, YY AND
C             IY SEE THE PACKAGE 'RAY'.
C NONE OF THE INPUT PARAMETERS ARE ALTERED.
C
C NO OUTPUT.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1990, JANUARY 22
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE RPAR4(IRAY,PAR1,PAR2,YL,Y,YY,IY,IEND)
C
      REAL PAR1,PAR2,YL(6),Y(35),YY(5)
      INTEGER IY(12),IEND,IRAY
C
C THIS SUBROUTINE IS CALLED AFTER FINISHING THE COMPUTATION OF A RAY.
C IT DEFINES THE STORAGE LOCATIONS YI(22) TO YI(25) OF THE COMMON BLOCK
C /INITC/.
C
C INPUT:
C     IRAY... INDEX OF THE RAY, I.E. THE NUMBER OF THE ALREADY COMPUTED
C             RAYS, I.E. THE OUTPUT FROM THE LAST INVOCATION OF RPAR2.
C     PAR1,PAR2... TAKE-OFF PARAMETERS OF THE RAY.
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.  FOR THE DEFINITION OF ARRAYS YL, Y, YY AND
C             IY SEE THE PACKAGE 'RAY'.
C     IEND... REASON OF THE TERMINATION OF THE COMPUTATION OF THE RAY
C             (SEE C.R.T.5.4).  FOR A DETAILED DESCRIPTION SEE
C             SUBROUTINE RAY (SUBROUTINE PACKAGE 'RAY').
C
C NO OUTPUT.
C
C COMMON BLOCK /INITC/ (SEE SUBROUTINE PACKAGE 'INIT'):
      INTEGER MSRFCA
      PARAMETER (MSRFCA=128)
      INTEGER ISB1I,ICB1I
      REAL YLI(6),YI(25),FSRFCA(MSRFCA)
      COMMON/INITC/ISB1I,ICB1I,YLI,YI,FSRFCA
C     ------------------------------------------------------------------
C     YI(22)..AREA OF THE ELEMENT OF THE RAY-PARAMETER SURFACE,
C             CORRESPONDING TO THE RAY,
C     YI(23),YI(24),YI(25)... COMPONENTS 11, 12, 22 OF THE SYMMETRIC
C             MATRIX INVERSE TO THE SPECIFIC MOMENT OF THE ELEMENT OF
C             THE RAY-PARAMETER SURFACE CORRESPONDING TO THE RAY, SEE
C             C.R.T.6.1.
C LOCATIONS YI(22) TO YI(25) OF THE COMMON BLOCK ARE DEFINED IN THIS
C SUBROUTINE, OTHER STORAGE LOCATIONS REMAIN UNCHANGED.
C
C COMMON BLOCK /RPARC/:
      INTEGER LURPAR
      REAL PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
      COMMON/RPARC/ LURPAR,PAR1L,PAR1M,PAR1D,PAR2L,PAR2M,PAR2D
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1990, JANUARY 22
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     NO AUXILIARY STORAGE LOCATIONS.
C
      IF(PAR1D.EQ.0.) THEN
        YI(22)=1.
        YI(23)=0.
      ELSE
        YI(22)=ABS(PAR1D)
        YI(23)=12./(PAR1D*PAR1D)
      END IF
      IF(PAR2D.EQ.0.) THEN
        YI(25)=0.
      ELSE
        YI(22)=YI(22)*ABS(PAR2D)
        YI(25)=12./(PAR2D*PAR2D)
      END IF
      YI(24)=0.
      RETURN
      END
C
C=======================================================================
C
